# No fancy boiler-plate, this is just a dummy empty-sub-class of
# Test::AgainstSchema for the sake of testing it.

package MyTestAgainstSchema;

use 5.008;
use strict;
use warnings;
use base qw(Test::AgainstSchema);

our $VERSION = '1.000';

1;
